/*
Copyright (C) 2005 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package orbites;

import java.awt.*;

/**
 * Reprsente une toile
 */
public class Etoile {
    
    private static final double mSoleil = 1.9887973E30; // masse du Soleil (en kg)
    private static final double rSoleil = 6.96E8; // rayon du Soleil (en m)
    private static final double mbolSoleil = 4.76;
    private static final double UA = 1.49597870691E11; // une unit astronomique (en m)
    private static final boolean activerImages = true;
    private Orbites orbites;
    private int imgdimx;
    private int imgdimy;
    private String nomEtoile, spectre;
    private double masse;
    private boolean masseConnue;
    private Color couleurEtoile;
    private double rayon;
    private double magnitude;
    private double distance;
    private Image imgEtoile;
    
    public Etoile(Orbites orbites) {
        this.orbites = orbites;
    }
    
    public Image getImage() {
        return(imgEtoile);
    }
    
    public String getSpectre() {
        return(spectre);
    }
    
    public double getRayon() {
        return(rayon);
    }
    
    public double getMasse() {
        return(masse);
    }
    
    public double getMagnitude() {
        return(magnitude);
    }
    
    public double getDistance() {
        return(distance);
    }
    
    /** luminosit en L_soleil */
    public double getLuminositeSolaire() {
        if (getSpectre() == null || getDistance() == 0)
            return(0); // on se sert du type spectral pour obtenir la temprature, pour obtenir la correction bolomtrique
        double lTm4 = Math.log(getTemperature())/Math.log(10) - 4;
        // formule empirique de la correction bolomtrique :
        double BC = -8.499*Math.pow(lTm4, 4) + 13.421*Math.pow(lTm4, 3) - 8.131*Math.pow(lTm4, 2) - 3.901*lTm4 - 0.438;
        double mAbs = getMagnitude() - 5*Math.log(getDistance())/Math.log(10) + 5;
        double mbolEtoile = mAbs + BC;
        return(Math.pow(2.512, mbolSoleil - mbolEtoile));
    }
    
    public void init(boolean activerImages, int imgdimx, int imgdimy) {
        this.imgdimx = imgdimx;
        this.imgdimy = imgdimy;
        
        if (activerImages && imgEtoile == null) {
            String suffixe;
            if ("1.2".compareTo(System.getProperty("java.version")) < 0)
                suffixe = ".png";
            else
                suffixe = ".gif";
            if ("pulsar".equals(spectre))
                imgEtoile = orbites.getImage(orbites.getDocumentBase(), "images/pulsar" + suffixe);
            else {
                String spec1 = null;
                if (spectre != null)
                    spec1 = spectre.substring(0, 1).toUpperCase();
                if (spec1 == null || "OBAFGKM".indexOf(spec1) == -1) {
                    System.err.println("imageEtoile: erreur: spectre inconnu: " + spectre);
                    imgEtoile = null;
                } else
                    imgEtoile = orbites.getImage(orbites.getDocumentBase(), "images/etoile" + spec1 + suffixe);
            }
        }
        if (rayon == 0) {
            if (spectre == null)
                rayon = 6.96E8; // si le spectre n'est pas prcis, on prend le rayon du soleil
            else if ("pulsar".equals(spectre))
                rayon = 1E4;
            else {
                double detoile = spectreVersDensite(spectre); // densit en kg/m^3
                rayon = Math.pow((3*masse)/(4*Math.PI*detoile), 1.0/3.0); // rayon toile en m
            }
        }
    }
    
    protected void lectureParametres(double periode1, double dgap1) throws NumberFormatException {
        nomEtoile = orbites.getParameter("nomEtoile");
        String smetoile = orbites.getParameter("masseEtoile");
        if (smetoile == null || "".equals(smetoile)) {
            masseConnue = false;
            if (periode1 == 0)
                throw new NumberFormatException(Messages.get("masse_et_periode_inconnues"));
            masse = (Math.pow(dgap1/UA, 3) / Math.pow(periode1/365.25, 2)) * mSoleil;
        } else {
            masseConnue = true;
            masse = Double.valueOf(smetoile).doubleValue() * mSoleil;
        }
        spectre = orbites.getParameter("spectreEtoile");
        if ("".equals(spectre))
            spectre = null;
        couleurEtoile = spectreVersCouleur(spectre);
        String srayon = orbites.getParameter("rayonEtoile");
        if (srayon != null && !"".equals(srayon) && !"0".equals(srayon))
            rayon = Double.valueOf(srayon).doubleValue() * rSoleil;
        else
            rayon = 0;
        String smagnitude = orbites.getParameter("magnitudeEtoile");
        if (smagnitude == null || "".equals(smagnitude))
            magnitude = 0;
        else
            magnitude = Double.valueOf(smagnitude).doubleValue();
        String sdistance = orbites.getParameter("distanceEtoile");
        if (sdistance == null || "".equals(sdistance))
            distance = 0;
        else
            distance = Double.valueOf(sdistance).doubleValue();
        String simage = orbites.getParameter("imageEtoile");
        if (simage != null && !simage.equals("")) {
            String suffixe;
            if ("1.2".compareTo(System.getProperty("java.version")) < 0)
                suffixe = ".png";
            else
                suffixe = ".gif";
            imgEtoile = orbites.getImage(orbites.getDocumentBase(), simage + suffixe);
        } else
            imgEtoile = null;
    }
    
    protected void majCroquis(Graphics g, double rapport, double r1, boolean activerImages) {
        int ir1 = (int)Math.round(r1*rapport+0.5);
        int id1 = (int)Math.round(2*r1*rapport+1);
        
        // on utilise seulemenent la plante la plus importante pour calculer la position
        Planete plMax = orbites.planeteImportante();
        double dgap = plMax.getDgap();
        double mplanete = plMax.getMasse();
        double eccentricite = plMax.getEccentricite();
        double omega = plMax.getOmega();
        double inc = plMax.getInc();
        double tpstheta = plMax.getTpstheta();
        double rorb2 = plMax.getRorb();
        double rorb1 = rorb2 * mplanete / masse;
        
        double p1 = rorb1 * (1 - Math.pow(eccentricite, 2));
        double tpstheta1 = tpstheta + Math.PI;
        // (rr1, tpstheta1 + omega) : coordonnes polaires de l'toile
        // (x1, y1) : coordonnes cartsiennes de l'toile
        double rr1 = p1 / (1 + eccentricite * Math.cos(tpstheta1));
        double x1 = rr1 * Math.cos(tpstheta1 + omega);
        double y1 = rr1 * Math.sin(tpstheta1 + omega) * Math.sin(inc);
        
        int ix1 = (int)Math.round(imgdimx/2 + x1*rapport) - ir1;
        int iy1 = (int)Math.round(imgdimy/2 - y1*rapport) - ir1;
        
        if (activerImages)
            imageEtoile(g, ix1, iy1, id1, id1);
        else
            dessinEtoile(g, ix1, iy1, id1, id1);
    }
    
    protected int afficherInfos(Graphics g, int post) {
        g.setColor(Color.white);
        if (nomEtoile == null || "".equals(nomEtoile))
            g.drawString(Messages.get("etoile") + " " + Messages.get("?"), 5, post+=12);
        else
            g.drawString(Messages.get("etoile") + " " + nomEtoile, 5, post+=12);
        if (!masseConnue)
            g.drawString(Messages.get("masse_etoile_calculee"), 5 , post+=12);
        //g.drawString("fps : " + joliFormattage(1000/temporisation) + " images/s", 5, post+=12);
        return(post);
    }
    
    protected void afficherParametres(Graphics g, double x1, double y1, double rapport, boolean affmasses) {
        if (affmasses) {
            int xl = (int)Math.round(imgdimx/2 + x1*rapport) + 10;
            int yl = (int)Math.round(imgdimy/2 - y1*rapport);
            g.drawString("M = " + joliFormattage(masse/mSoleil) + " " + Messages.get("Msol"), xl, yl);
        }
    }
    
    // on donne les dimensions de l'toile, mais le dessin entier sortira de ce rectangle
    protected void dessinEtoile(Graphics g, int x, int y, int longueur, int hauteur) {
        int rayon = longueur/2;
        int cx = x + rayon;
        int cy = y + rayon;
        int rouge = couleurEtoile.getRed();
        int vert = couleurEtoile.getGreen();
        int bleu = couleurEtoile.getBlue();
        Color couleur;
        
        double ratio;
        for (int i=rayon; i>0; i-=2) {
            ratio = i*1.0/rayon;
            couleur = new Color(255 - (int)Math.round((255-rouge)*ratio),
                255 - (int)Math.round((255-vert)*ratio),
                255 - (int)Math.round((255-bleu)*ratio));
            g.setColor(couleur);
            g.fillOval(cx-i, cy-i, i*2, i*2);
        }
        /*
        couleur = new Color(rouge*3/4, vert*3/4, bleu*3/4);
        g.setColor(couleur);
        g.drawOval(x-rayon/3, y-rayon/3, rayon*8/3, rayon*8/3);
        g.drawOval(x-rayon/3-1, y-rayon/3-1, rayon*8/3+2, rayon*8/3+2);
        
        g.drawOval(cx-rayon*2, cy-rayon*2, rayon*4, rayon*4);
        
        int[] tx = new int[3];
        int[] ty = new int[3];
        
        tx[0] = cx-1;  ty[0] = y-rayon/3;
        tx[1] = cx;    ty[1] = y-rayon*3;
        tx[2] = cx+1;  ty[2] = y-rayon/3;
        g.fillPolygon(tx, ty, 3);
        
        tx[0] = cx+4*rayon/3;  ty[0] = cy-1;
        tx[1] = cx+rayon*4;    ty[1] = cy;
        tx[2] = cx+4*rayon/3;  ty[2] = cy+1;
        g.fillPolygon(tx, ty, 3);
        
        tx[0] = cx-1;  ty[0] = cy+4*rayon/3;
        tx[1] = cx;    ty[1] = cy+rayon*4;
        tx[2] = cx+1;  ty[2] = cy+4*rayon/3;
        g.fillPolygon(tx, ty, 3);
        
        tx[0] = x-rayon/3;  ty[0] = cy-1;
        tx[1] = x-rayon*3;    ty[1] = cy;
        tx[2] = x-rayon/3;  ty[2] = cy+1;
        g.fillPolygon(tx, ty, 3);
        */
    }
    
    // on donne les dimensions de l'toile, mais l'image sortira de ce rectangle
    protected void imageEtoile(Graphics g, int x, int y, int longueur, int hauteur) {
        if (imgEtoile == null) {
            g.setColor(couleurEtoile);
            g.fillOval(x, y, longueur, hauteur);
            return;
        }
        double rapportDisque;
        if ("pulsar".equals(spectre))
            rapportDisque = 50; // en fait le disque est invisible pour un pulsar
        else
            rapportDisque = 167;
        int w2 = (int)Math.round(longueur * (imgEtoile.getWidth(orbites)/rapportDisque)+0.5); // vieilles images: 100
        int h2 = (int)Math.round(longueur * (imgEtoile.getHeight(orbites)/rapportDisque)+0.5);
        int x2 = x + (longueur - w2)/2;
        int y2 = y + (hauteur - h2)/2;
        g.drawImage(imgEtoile, x2, y2, w2, h2, orbites);
    }
    
    protected static Color spectreVersCouleur(String spectre) {
        // d'aprs "What color are the stars?" par Mitchell Charity
        // http://www.vendian.org/mncharity/dir3/starcolor/
        // on pourrait faire plus prcis avec une table comme celle 
        // http://www.vendian.org/mncharity/dir3/starcolor/UnstableURLs/starcolors.html
        if ("pulsar".equals(spectre)) {
            // couleur d'un pulsar ?
            return(new Color(255, 255, 255));
        }
        String spec1;
        if (spectre != null && spectre.length() > 0)
            spec1 = spectre.substring(0, 1);
        else
            spec1 = null;
        /*
            Sp5(V)  r   g   b     rgb
              O     155 176 255  #9bb0ff  
              B     170 191 255  #aabfff  
              A     202 215 255  #cad7ff  
              F     248 247 255  #f8f7ff  
              G     255 244 234  #fff4ea  
              K     255 210 161  #ffd2a1  
              M     255 204 111  #ffcc6f  
        */
        Color c;
        if ("O".equals(spec1))
            c = new Color(155, 176, 255);
        else if ("B".equals(spec1))
            c = new Color(170, 191, 255);
        else if ("A".equals(spec1))
            c = new Color(202, 215, 255);
        else if ("F".equals(spec1))
            c = new Color(248, 247, 255);
        else if ("G".equals(spec1))
            c = new Color(255, 244, 234);
        else if ("K".equals(spec1))
            c = new Color(255, 210, 161);
        else if ("M".equals(spec1))
            c = new Color(255, 204, 111);
        else {
            System.err.println("spectre non trouv dans la table: " + spectre);
            c = new Color(255, 255, 255);
        }
        return(c);
    }
    
    // densit en kg/m^3
    // pifomtrique mais mieux que rien !
    protected static double spectreVersDensite(String spectre) {
        // d'aprs
        // http://cassfos02.ucsd.edu/physics/ph7/Stars.html
        String spec1;
        if (spectre != null)
            spec1 = spectre.substring(0, 1);
        else
            spec1 = null;
        double dsoleil = mSoleil/((4./3.)*Math.PI*Math.pow(6.96E8,3));
        double d;
        if ("O".equals(spec1))
            d = 3.11;
        else if ("B".equals(spec1))
            d = 1.57;
        else if ("A".equals(spec1))
            d = 1.14;
        else if ("F".equals(spec1))
            d = 1;
        else if ("G".equals(spec1))
            d = 1;
        else if ("K".equals(spec1))
            d = 0.96;
        else if ("M".equals(spec1))
            d = 0.69;
        else {
            System.err.println("spectre non trouv dans la table: " + spectre);
            d = 1;
        }
        return(d * dsoleil);
    }
    
    public double getTemperature() {
        return(spectreVersTemperature(spectre));
    }
    
    protected static double spectreVersTemperature(String spectre) {
        if ("pulsar".equals(spectre))
            return(1E6); // trs approximatif !
        
        // sur la squence principale
        // cf http://www.earth.uni.edu/~morgan/astro/course/Notes/section2/spectraltemps.html
        String[] tspec2 = {"O5", "O6", "O7", "O8", "O9", "B0", "B1", "B2", "B3", "B5", "B6", "B7",
            "B8", "B9", "A0", "A1", "A2", "A3", "A4", "A5", "A7", "F0", "F2", "F3", "F5", "F6",
            "F7", "F8", "G0", "G1", "G2", "G5", "G8", "K0", "K1", "K2", "K3", "K4", "K5", "K7",
            "M0", "M1", "M2", "M3", "M4", "M5", "M6", "M7", "M8"};
        double[] ttemp = {54000, 45000, 43300, 40600, 37800, 29200, 23000, 21000, 17600, 15200,
            14300, 13500,
            12300, 11400, 9600, 9330, 9040, 8750, 8480, 8310, 7920, 7350, 7050, 6850, 6700, 6550,
            6400, 6300, 6050, 5930, 5800, 5660, 5440, 5240, 5110, 4960, 4800, 4600, 4400, 4000,
            3750, 3700, 3600, 3500, 3400, 3200, 3100, 2900, 2700};
        if (spectre != null && spectre.length() > 1) {
            String spec2 = spectre.substring(0, 2);
            for (int i=0; i<tspec2.length; i++)
                if (tspec2[i].equals(spec2))
                    return(ttemp[i]);
        }
        String spec1;
        if (spectre != null)
            spec1 = spectre.substring(0, 1);
        else
            spec1 = null;
        if ("O".equals(spec1))
            return(35000);
        else if ("B".equals(spec1))
            return(13500);
        else if ("A".equals(spec1))
            return(8100);
        else if ("F".equals(spec1))
            return(6500);
        else if ("G".equals(spec1))
            return(5400);
        else if ("K".equals(spec1))
            return(4000);
        else if ("M".equals(spec1))
            return(2600);
        else {
            System.err.println("spectre non trouv dans la table: " + spectre);
            return(5400);
        }
    }
    
	// mthode d'affichage de double issue du Calcotron
    protected static String joliFormattage(double v) {
        int precision = 3;
        int limitesci = 3; // dcimales avant de passer en mode sci
        
        if (v == 0)
            return("0");
        String sj = ""+v;
        if ("Infinity".equals(sj) || "-Infinity".equals(sj) || "NaN".equals(sj))
            return(""+v);
        String signe;
        if (v >= 0)
            signe = "";
        else {
            v = -v;
            signe = "-";
        }
        int exposant = (int)Math.floor(Math.log(v)/Math.log(10));
        double exparr = Math.pow(10, precision);
        double mantisse = v / Math.pow(10, exposant);
        mantisse = Math.rint(mantisse*exparr)/exparr;
        if (mantisse >= 10) {
            exposant += 1;
            mantisse = v / Math.pow(10, exposant);
            mantisse = Math.rint(mantisse*exparr)/exparr;
        }
        String sv = "";
        int chiffre = (int)Math.floor(mantisse);
        sv += chiffre + ".";
        for (int i=0; i<precision-1; i++) {
            mantisse = (mantisse - chiffre)*10;
            double exparr2 = Math.pow(10, precision-i);
            mantisse = Math.rint(mantisse*exparr)/exparr;
            chiffre = (int)Math.floor(mantisse);
            sv += chiffre;
        }
        mantisse = (mantisse - chiffre)*10;
        chiffre = (int)Math.round(mantisse);
        sv += chiffre;
        char dernier = sv.charAt(sv.length()-1);
        while (dernier == '0') {
            sv = sv.substring(0, sv.length()-1);
            dernier = sv.charAt(sv.length()-1);
        }
        if (exposant != 0) {
            if (exposant > 0 && exposant < limitesci) {
                String sd = sv.substring(0,1);
                if (sv.length()-2 < exposant) {
                    // transformation de 3.2E2 en 320
                    if (sv.length() > 2)
                        sd += sv.substring(2,sv.length());
                    for (int i=0; i<exposant-(sv.length()-2); i++)
                        sd += '0';
                } else {
                    // transformation de 3.245E2 en 324.5
                    if (sv.length() < exposant+2)
                        sd += sv.substring(2);
                    else
                        sd += sv.substring(2,exposant+2) + "." + sv.substring(exposant+2);
                }
                sv = sd;
            } else if (exposant < 0 && exposant > -limitesci) {
                // transformation de 3.2E-2 en 0.032
                String sd = "0.";
                for (int i=0; i<-exposant-1; i++)
                    sd += '0';
                sv = sd + sv.charAt(0) + sv.substring(2);
            } else {
                if (sv.charAt(sv.length()-1) == '.')
                    sv = sv.substring(0, sv.length()-1);
                sv += ".10^" + exposant;
            }
        }
        if (sv.charAt(sv.length()-1) == '.')
            sv = sv.substring(0, sv.length()-1);
        
        sv = signe + sv;
        
        return(sv);
    }
}
