/*
Copyright (C) 2005 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package orbites;

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.MemoryImageSource;
import java.lang.reflect.Method;

/**
 * Reprsente l'affichage d'un systme plantaire
 *
 * Dimensions: 300x300
 *
 * Paramtres de l'applet (X est le numro de la plante,  partir de 1)
 * nomEtoile - nom de l'toile
 * masseEtoile - masse de l'toile (en masse solaire)
 * spectreEtoile - spectre de l'toile
 * magnitudeEtoile - magnitude visuelle de l'toile
 * distanceEtoile - distance de l'toile en pc
 * nomX - nom de la plante
 * aX - demi-grand axe (en UA)
 * TX - priode (en jours)
 * eX - excentricit de l'orbite
 * iX - inclinaison de l'orbite (vide si inconnu) (en degrs)
 * msiniX - M.sini (en masse de Jupiter)
 */
public class Orbites extends Applet implements Runnable, ItemListener, KeyListener {
    
    private static final double UA = 1.49597870691E11; // une unit astronomique (en m)
    private static final boolean activerImages = true;
    private int n = 300; // nombre d'itrations par orbite pour la plante 1
    private int temporisation = 100; // temporisation entre 2 images en ms
    private int imgdimx = 300;
    private int imgdimy = 300;
    private ImageCanvas ic;
    private long chrono;
    private Etoile etoile;
    private int npl;
    private Planete[] planetes;
    private double tps;
    private int itps;
    private boolean fin;
    private Thread monThread;
    private boolean tailleReelle, distanceReelle, inclinaisonNulle;
    private boolean affdga, affinc, affmasses, affecc, afforb, affomega, afforbterre, affzonehab;
    private Checkbox checkDistance, checkTaille, checkInclinaisonNulle;
    private Checkbox checkdga, checkinc, checkmasses, checkecc, checkorb, checkomega, checkorbterre, checkzonehab;
    private double dhabmin, dhabmax; // d.g.a. min et max pour la zone d'habitabilit
    private Image imgzonehab1, imgzonehab2;
    private int wminimgzonehab;
    private double rapport; // rapport des distances en pixels/m
    private int plsel = -1; // plante slectionne
    private double mindgap, maxdgap; // mini et maxi d.g.a.
    private int pldgamax = 0; // plante dont le d.g.a. est maximum
    private double maxr = 0; // rayon maximum des plantes
    private boolean incConnue; // true si l'inclinaison d'une plante est connue
    private int zoom;
    private double rotationTotale;
    
    public void init() {
        fin = false;
        try {
            lectureParametres();
        } catch (NumberFormatException ex) {
            System.err.println("NumberFormatException: " + ex.getMessage());
            fin = true;
            add( new Label(Messages.get("erreur_lecture") + " " + ex.getMessage()) );
            return;
        }
        
        Dimension tailleApplet = getSize();
        if (tailleApplet != null) {
            imgdimx = tailleApplet.width - 200;
            imgdimy = tailleApplet.height - 40;
        }
        
        etoile.init(activerImages, imgdimx, imgdimy);
        
        incConnue = false;
        for (int ipl=0; ipl<npl; ipl++)
            if (planetes[ipl].getIncConnue())
                incConnue = true;
        
        mindgap = 0;
        maxdgap = 0;
        for (int ipl=0; ipl<npl; ipl++) {
            if (planetes[ipl].getDgap() > maxdgap) {
                maxdgap = planetes[ipl].getDgap();
                pldgamax = ipl;
            }
            if (mindgap == 0 || planetes[ipl].getDgap() < mindgap)
                mindgap = planetes[ipl].getDgap();
        }
        calculZoneHab();
        if (dhabmin != 0 && dhabmin < mindgap)
            mindgap = dhabmin;
        
        for (int ipl=0; ipl<npl; ipl++) {
            planetes[ipl].init(etoile, activerImages, dhabmin, dhabmax, imgdimx, imgdimy);
        }
        
        maxr = 0;
        for (int ipl=0; ipl<npl; ipl++) {
            if (planetes[ipl].getRpl() > maxr)
                maxr = planetes[ipl].getRpl();
        }
        
        if (activerImages) {
            if (etoile.getImage() != null) {
                MediaTracker tracker = new MediaTracker(this);
                if (etoile.getImage() != null)
                    tracker.addImage(etoile.getImage(), 0);
                for (int ipl=0; ipl<npl; ipl++)
                    tracker.addImage(planetes[ipl].getImage(), 0);
                try {
                    tracker.waitForAll();
                } catch (InterruptedException ex) {
                    System.err.println("InterruptedException: " + ex.getMessage());
                }
            }
        }
        
        setLayout(new BorderLayout());
        setBackground(new Color(0, 34, 62));
        setForeground(Color.white);
        
        ic = new ImageCanvas(this, imgdimx, imgdimy);
        add(ic, BorderLayout.CENTER);
        
        Panel droite = new Panel(new BorderLayout());
        int nbel = 7;
        if (incConnue)
            nbel++;
        if (maxdgap > UA)
            nbel++;
        if (maxdgap > dhabmin)
            nbel++;
        Panel hautdroite = new Panel(new GridLayout(nbel, 1));
        hautdroite.add(new Label(Messages.get("afficher")));
        afforb = false;
        checkorb = new Checkbox(Messages.get("orbite"), afforb);
        checkorb.addItemListener(this);
        fixKeyListener(checkorb);
        hautdroite.add(checkorb);
        affdga = false;
        checkdga = new Checkbox(Messages.get("demi_grand-axe"), affdga);
        checkdga.addItemListener(this);
        fixKeyListener(checkdga);
        hautdroite.add(checkdga);
        affinc = false;
        if (incConnue) {
            checkinc = new Checkbox(Messages.get("inclinaison"), affinc);
            checkinc.addItemListener(this);
            fixKeyListener(checkinc);
            hautdroite.add(checkinc);
        }
        affmasses = false;
        checkmasses = new Checkbox(Messages.get("masses"), affmasses);
        checkmasses.addItemListener(this);
        fixKeyListener(checkmasses);
        hautdroite.add(checkmasses);
        affecc = false;
        checkecc = new Checkbox(Messages.get("excentricite"), affecc);
        checkecc.addItemListener(this);
        fixKeyListener(checkecc);
        hautdroite.add(checkecc);
        affomega = false;
        checkomega = new Checkbox(Messages.get("omega"), affomega);
        checkomega.addItemListener(this);
        fixKeyListener(checkomega);
        hautdroite.add(checkomega);
        afforbterre = false;
        if (maxdgap > UA) {
            checkorbterre = new Checkbox(Messages.get("orbite_1_UA"), afforbterre);
            checkorbterre.addItemListener(this);
            fixKeyListener(checkorbterre);
            hautdroite.add(checkorbterre);
        }
        affzonehab = false;
        if (maxdgap > dhabmin) {
            checkzonehab = new Checkbox(Messages.get("zone_habitable"), affzonehab);
            checkzonehab.addItemListener(this);
            fixKeyListener(checkzonehab);
            hautdroite.add(checkzonehab);
        }
        droite.add(hautdroite, BorderLayout.NORTH);
        add(droite, BorderLayout.EAST);
        
        Panel bas = new Panel(new GridLayout(1, incConnue ? 3 : 2));
        distanceReelle = false;
        tailleReelle = (maxdgap < UA/10);
        checkDistance = new Checkbox(Messages.get("distance_reelle"), distanceReelle);
        checkDistance.addItemListener(this);
        fixKeyListener(checkDistance);
        bas.add(checkDistance);
        checkTaille = new Checkbox(Messages.get("taille_reelle"), tailleReelle);
        checkTaille.addItemListener(this);
        fixKeyListener(checkTaille);
        bas.add(checkTaille);
        if (incConnue) {
            checkInclinaisonNulle = new Checkbox(Messages.get("inclinaison_0"), inclinaisonNulle);
            checkInclinaisonNulle.addItemListener(this);
            fixKeyListener(checkInclinaisonNulle);
            bas.add(checkInclinaisonNulle);
        }
        add(bas, BorderLayout.SOUTH);
        
        // pour viter un bug de Tiger, on doit rpter les couleurs pour chaque composant
        modifCouleurs(hautdroite);
        modifCouleurs(checkorb);
        modifCouleurs(checkdga);
        if (checkinc != null)
            modifCouleurs(checkinc);
        modifCouleurs(checkmasses);
        modifCouleurs(checkecc);
        modifCouleurs(checkomega);
        if (checkorbterre != null)
            modifCouleurs(checkorbterre);
        if (checkzonehab != null)
            modifCouleurs(checkzonehab);
        modifCouleurs(bas);
        modifCouleurs(checkDistance);
        modifCouleurs(checkTaille);
        if (checkInclinaisonNulle != null)
            modifCouleurs(checkInclinaisonNulle);
        
        zoom = 0;
        addKeyListener(this);
        ic.addKeyListener(this);
        
        validate();
        setVisible(true);
        ic.requestFocus();
    }
    
    protected void modifCouleurs(Component c) {
        c.setBackground(new Color(0, 34, 62));
        c.setForeground(Color.white);
    }
    
    protected void lectureParametres() throws NumberFormatException {
        npl = 0;
        for (int ipar=1; getParameter("a"+ipar) != null; ipar++) {
            if (!"".equals(getParameter("a"+ipar)))
                npl++;
        }
        if (npl == 0)
            System.err.println("Aucune plante spcifie !");
        planetes = new Planete[npl];
        int ipl = 0;
        for (int ipar=1; ipl<npl; ipar++) {
            if (!"".equals(getParameter("a"+ipar))) {
                planetes[ipl] = new Planete(this);
                planetes[ipl].lectureParametres(ipar);
                ipl++;
            }
        }
        plsel = npl/2;
        etoile = new Etoile(this);
        etoile.lectureParametres(planetes[0].getPeriode(), planetes[0].getDgap());
    }
    
    public void start() {
        monThread = new Thread(this);
        monThread.start();
    }
    
    public void run() {
        itps = 0;
        long t1, dt;
        long totaldt = 0;
        int nbimtest = 9;
        long mindt = temporisation;
        while (!fin) {
            t1 = System.currentTimeMillis();
            chrono = -1;
            ic.repaint();
            try {
                Thread.sleep(temporisation);
            } catch (InterruptedException ex) {
            }
            
            itps++;
            if (itps == n)
                itps = 0;
            tps = (itps * 2 * Math.PI) / n;
            
            int nprecision = 10;
            //tpstheta += (2*Math.PI/n) * Math.pow(rorb2,2) * Math.sqrt(1 - Math.pow(eccentricite, 2)) / Math.pow(rr2, 2);
            // intgration de l'angle theta: plusieurs itrations pour plus de prcision
            for (int ipl=0; ipl<npl; ipl++) {
                double interv = 2*Math.PI / (n * nprecision);
                interv /= planetes[ipl].getPeriode()/planetes[plsel].getPeriode();
                planetes[ipl].iteration(interv);
            }
            
            if (chrono == -1)
                dt = 0;
            else
                dt = temporisation - (chrono - t1);
            if (dt < mindt)
                mindt = dt;
            totaldt += dt;
            if (itps/nbimtest == itps/(nbimtest*1.0)) {
                if (nbimtest < 18 && totaldt > nbimtest*1.0*temporisation*2./3 && mindt > 40) {
                    n = (n/100+1)*100; // plus d'images par sec
                    itps = (itps/nbimtest)*(nbimtest+3);
                    nbimtest += 3;
                    temporisation = 900/nbimtest;
                } else if (nbimtest > 3 && (totaldt < nbimtest*1.0*temporisation/5 || mindt < 15)) {
                    n = (n/100-1)*100; // moins d'images par sec
                    itps = (itps/nbimtest)*(nbimtest-3);
                    nbimtest -= 3;
                    temporisation = 900/nbimtest;
                }
                totaldt = 0;
                mindt = temporisation;
            }
        }
    }
    
    public void topChrono() {
        chrono = System.currentTimeMillis();
    }
    
    public void stop() {
        fin = true;
    }
    
    public void itemStateChanged(ItemEvent e) {
        if (e.getItemSelectable() == checkDistance)
            distanceReelle = (e.getStateChange() == ItemEvent.SELECTED);
        else if (e.getItemSelectable() == checkTaille)
            tailleReelle = (e.getStateChange() == ItemEvent.SELECTED);
        else if (e.getItemSelectable() == checkInclinaisonNulle) {
            inclinaisonNulle = (e.getStateChange() == ItemEvent.SELECTED);
            for (int ipl=0; ipl<npl; ipl++)
                planetes[ipl].setInclinaisonNulle(inclinaisonNulle);
        } else if (e.getItemSelectable() == checkorb)
            afforb = (e.getStateChange() == ItemEvent.SELECTED);
        else if (e.getItemSelectable() == checkdga)
            affdga = (e.getStateChange() == ItemEvent.SELECTED);
        else if (e.getItemSelectable() == checkinc)
            affinc = (e.getStateChange() == ItemEvent.SELECTED);
        else if (e.getItemSelectable() == checkmasses)
            affmasses = (e.getStateChange() == ItemEvent.SELECTED);
        else if (e.getItemSelectable() == checkecc)
            affecc = (e.getStateChange() == ItemEvent.SELECTED);
        else if (e.getItemSelectable() == checkomega)
            affomega = (e.getStateChange() == ItemEvent.SELECTED);
        else if (e.getItemSelectable() == checkorbterre)
            afforbterre = (e.getStateChange() == ItemEvent.SELECTED);
        else if (e.getItemSelectable() == checkzonehab)
            affzonehab = (e.getStateChange() == ItemEvent.SELECTED);
    }
    
    private void fixKeyListener(Checkbox chk) {
        chk.addKeyListener(new KeyAdapter() {
            public void keyTyped(KeyEvent e) {
                Orbites.this.keyTyped(e);
            }
        });
    }
    
    public void keyTyped(KeyEvent e) {
        if (e.getKeyChar() == '+')
            zoom++;
        else if (e.getKeyChar() == '-')
            zoom--;
    }
    
    public void keyPressed(KeyEvent e) {
    }
    
    public void keyReleased(KeyEvent e) {
    }
    
    public void rotation(Point pt1, Point pt2) {
        int dy = pt2.y - pt1.y;
        double rotrad = dy / 100.0;
        if (rotationTotale + rotrad <= 0 && rotationTotale + rotrad > -Math.PI) {
            rotationTotale += rotrad;
            for (int ipl=0; ipl<npl; ipl++)
                planetes[ipl].rotation(rotrad);
        }
    }
    
    protected void initGraphics(Graphics g) {
        g.setFont(new Font(getFont().getName(), Font.BOLD, 10));
        
        if ("1.2".compareTo(System.getProperty("java.version")) <= 0) {
            // Java 1.2+
            // pour rester compatible on utilise la reflection
            /* en clair a donne:
                Graphics2D g2d = (Graphics2D)g;
                g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                    RenderingHints.VALUE_ANTIALIAS_ON);
                g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
                    RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
            */
            try {
                // et voil la version crypte :)
                Class RenderingHints = Class.forName("java.awt.RenderingHints");
                Object KEY_ANTIALIASING = RenderingHints.getField("KEY_ANTIALIASING").get(null);
                Object KEY_TEXT_ANTIALIASING = RenderingHints.getField("KEY_TEXT_ANTIALIASING").get(null);
                Object VALUE_ANTIALIAS_ON = RenderingHints.getField("VALUE_ANTIALIAS_ON").get(null);
                Object VALUE_TEXT_ANTIALIAS_ON = RenderingHints.getField("VALUE_TEXT_ANTIALIAS_ON").get(null);
                Class Graphics2D = Class.forName("java.awt.Graphics2D");
                Class[] parameterTypes = new Class[2];
                parameterTypes[0] = Class.forName("java.awt.RenderingHints$Key");
                parameterTypes[1] = Object.class;
                Method setRenderingHint = Graphics2D.getMethod("setRenderingHint", parameterTypes);
                Object[] args = new Object[2];
                args[0] = KEY_ANTIALIASING;
                args[1] = VALUE_ANTIALIAS_ON;
                setRenderingHint.invoke(g, args);
                args[0] = KEY_TEXT_ANTIALIASING;
                args[1] = VALUE_TEXT_ANTIALIAS_ON;
                setRenderingHint.invoke(g, args);
            } catch (Exception ex) {
                System.err.println("Orbite: " + ex.getClass().getName() + ": " + ex.getMessage());
            }
        }
    }
    
    protected void selection(int x, int y) {
        double mindist = -1;
        int isel = -1;
        for (int ipl=0; ipl<npl; ipl++) {
            double dist = planetes[ipl].distsel(x, y);
            if (mindist == -1 || dist < mindist) {
                mindist = dist;
                isel = ipl;
            }
        }
        plsel = isel;
    }
    
    // fonction transformant les d.g.a. quand distanceReelle == false
    private double nouvelleDistance(double dga) {
        if (distanceReelle)
            return(dga);
        else if (tailleReelle) {
            double r1;
            if ("pulsar".equals(etoile.getSpectre()))
                r1 = etoile.getRayon()*5E3;
            else
                r1 = etoile.getRayon();
            if (mindgap == maxdgap)
                return(r1 * (3 + (dga - mindgap) / mindgap));
            else
                return((r1*3) * (0.4 + 0.6 * (Math.log(dga) - Math.log(mindgap)) / (Math.log(maxdgap) - Math.log(mindgap)) ) );
        } else {
            if (mindgap == maxdgap)
                return(dga);
            else
                return(maxdgap * (0.3 + 0.7 * (Math.log(dga) - Math.log(mindgap)) / (Math.log(maxdgap) - Math.log(mindgap)) ) );
        }
    }
    
    /** plante utilise pour calculer la position de l'toile **/
    protected Planete planeteImportante() {
        int imax = -1;
        double v;
        double maxv = 0;
        for (int ipl=0; ipl<npl; ipl++) {
            v = planetes[ipl].getMasse() / Math.pow(planetes[ipl].getDgap(), 2);
            if (v > maxv) {
                imax = ipl;
                maxv = v;
            }
        }
        if (imax == -1)
            return(null);
        else
            return(planetes[imax]);
    }
    
    protected void majCroquis(Graphics g) {
        for (int ipl=0; ipl<npl; ipl++)
            planetes[ipl].setRorb( nouvelleDistance(planetes[ipl].getDgap()) );
        double maxrorb = nouvelleDistance(maxdgap) * Math.pow(1.5, -zoom);
        
        double r1;
        if (tailleReelle) {
            if ("pulsar".equals(etoile.getSpectre()))
                r1 = etoile.getRayon()*5E3;
            else
                r1 = etoile.getRayon();
        } else
            r1 = maxrorb/10;
        double maxr2;
        if (tailleReelle)
            maxr2 = maxr;
        else
            maxr2 = r1/3;
        rapport = imgdimx / (2 * (maxrorb + maxr2*2 + maxrorb*planetes[pldgamax].getEccentricite()));
        
        double xsel=0, ysel=0;
        if (plsel != -1) {
            double[] pos = planetes[plsel].calculPos();
            xsel = pos[0];
            ysel = pos[1];
        }
        
        g.setColor(Color.black);
        g.fillRect(0, 0, imgdimx-1, imgdimy-1);
        
        if (affzonehab)
            dessinZoneHab(g, false);
        
        if (afforb && plsel != -1)
            planetes[plsel].dessinOrbite(g, rapport, false);
        
        int post = etoile.afficherInfos(g, 0);
        if (plsel != -1)
            planetes[plsel].afficherInfos(g, post);
        else
            afficherInfos(g, post);
        
        for (int ipl=0; ipl<npl; ipl++) {
            if (Math.sin(planetes[ipl].getTpstheta() + planetes[ipl].getOmega()) > 0)
                planetes[ipl].majCroquis(g, rapport, maxrorb, r1, tailleReelle, activerImages);
        }
        
        if (afforbterre)
            dessinOrbiteTerre(g, false);
        
        etoile.majCroquis(g, rapport, r1, activerImages);
        
        if (afforbterre)
            dessinOrbiteTerre(g, true);
        
        if (affzonehab)
            dessinZoneHab(g, true);
        
        if (afforb && plsel != -1)
            planetes[plsel].dessinOrbite(g, rapport, true);
        
        if (plsel != -1) {
            planetes[plsel].afficherParametres(g, xsel, ysel, rapport, affdga, affecc, affinc, affmasses, affomega);
            etoile.afficherParametres(g, 0, 0, rapport, affmasses);
        }
        
        for (int ipl=0; ipl<npl; ipl++) {
            if (Math.sin(planetes[ipl].getTpstheta() + planetes[ipl].getOmega()) <= 0)
                planetes[ipl].majCroquis(g, rapport, maxrorb, r1, tailleReelle, activerImages);
        }
        
        g.setColor(Color.black);
        g.drawRect(0, 0, imgdimx - 1, imgdimx - 1);
    }
    
    protected void afficherInfos(Graphics g, int post) {
        g.setColor(Color.white);
        g.drawString(Messages.get("cliquer"), 5, post+=12);
    }
    
    protected void dessinOrbiteTerre(Graphics g, boolean devant) {
        // on simplifie avec une orbite circulaire  1 UA (eccentricite=0)
        g.setColor(Color.cyan);
        int ndo = 100;
        double dotheta;
        double dga2 = nouvelleDistance(UA);
        for (int ido=0; ido<ndo; ido++) {
            dotheta = (ido * 2*Math.PI) / ndo;
            double x = dga2 * Math.cos(dotheta);
            //double y = dga2 * Math.sin(dotheta) * Math.sin(planetes[0].getInc());
            double y = dga2 * Math.sin(dotheta);
            if ((devant && y < 0) || (!devant && y >= 0)) {
                int ix = (int)Math.round(imgdimx/2 + x*rapport);
                int iy = (int)Math.round(imgdimy/2 - y*rapport);
                g.drawRect(ix, iy, 1, 1);
            }
        }
    }
    
    protected void calculZoneHab() {
        if (etoile.getSpectre() == null) {
            dhabmax = 0;
            dhabmin = 0;
            return;
        }
        if (etoile.getMagnitude() == 0 || etoile.getDistance() == 0) {
            //double refrac = 0.4; // pouvoir rfractaire de la plante (A dans l'quation)
            //double serre = 1.15; // effet de serre (G dans l'quation)
            // ( dfaut, on prend ici les valeurs pour la Terre)
            double tetoile = etoile.getTemperature(); // trs approximatif
            // Tpl = Tet*(Ret/2d)^1/2 * (1-A)^1/4 * G
            // d = Tet^2 * Ret * (1-A)^1/2 * G^2 / (2 * Tpl^2)
            // les tempratures prisent ici correspondent aux orbite de Vnus et Mars
            double Tmin = 225; // limite minimum = 169 K ?
            double Tmax = 329; // limite maximum = 269 K ?
            //double tmp = tetoile*tetoile * ret * Math.sqrt(1-refrac) * serre*serre;
            double tmp = tetoile*tetoile * etoile.getRayon(); // rayon calcul approximativement
            // temp min = distance max
            dhabmax = tmp / (2 * Tmin*Tmin); // d.g.a. limite en m
            dhabmin = tmp / (2 * Tmax*Tmax);
        } else {
            // mthode plus prcise base sur la magnitude et la distance de l'toile
            double facteur = Math.sqrt(etoile.getLuminositeSolaire());
            dhabmax = 1.524 * UA * facteur; // d.g.a. limite en m (base sur le dga de Mars)
            dhabmin = 0.723 * UA * facteur; // base sur le dga de Vnus
        }
    }
    
    protected void dessinZoneHab(Graphics g, boolean devant) {
        
        /*
        avec des limites traces par 2 ovales
        g.setColor(Color.cyan);
        // coord ovales limites pour dhabmax
        double x1 = -nouvelleDistance(dhabmax);
        double x2 = nouvelleDistance(dhabmax);
        double y1 = -nouvelleDistance(dhabmax) * Math.sin(inc);
        double y2 = nouvelleDistance(dhabmax) * Math.sin(inc);
        // coord en pixels pour d1
        int ix1 = (int)Math.round(imgdimx/2 + x1*rapport);
        int iy1 = (int)Math.round(imgdimy/2 + y1*rapport);
        int ix2 = (int)Math.round(imgdimx/2 + x2*rapport);
        int iy2 = (int)Math.round(imgdimy/2 + y2*rapport);
        g.drawOval(ix1, iy1, ix2-ix1, iy2-iy1);
        
        // coord ovales limites pour dhabmin
        x1 = -nouvelleDistance(dhabmin);
        x2 = nouvelleDistance(dhabmin);
        y1 = -nouvelleDistance(dhabmin) * Math.sin(inc);
        y2 = nouvelleDistance(dhabmin) * Math.sin(inc);
        // coord en pixels pour d2
        ix1 = (int)Math.round(imgdimx/2 + x1*rapport);
        iy1 = (int)Math.round(imgdimy/2 + y1*rapport);
        ix2 = (int)Math.round(imgdimx/2 + x2*rapport);
        iy2 = (int)Math.round(imgdimy/2 + y2*rapport);
        g.drawOval(ix1, iy1, ix2-ix1, iy2-iy1);
        */
        int w = (int)Math.round(2*nouvelleDistance(dhabmax)*rapport);
        int wimg;
        if (w > imgdimx)
            wimg = imgdimx;
        else
            wimg = w;
        int h = (int)Math.round(2*nouvelleDistance(dhabmax)*rapport);
        int himg;
        if (h > imgdimy)
            himg = imgdimy;
        else
            himg = h;
        if (himg != (himg / 2) * 2)
            himg = (himg / 2) * 2;
        int wmin = (int)Math.round(2*nouvelleDistance(dhabmin)*rapport);
        if (wmin > imgdimx)
            return;
        if (wminimgzonehab != wmin) {
            int hmin = (int)Math.round(2*nouvelleDistance(dhabmin)*rapport);
            int[] pix = new int[wimg * himg / 2];
            int index = 0;
            int rvb = 0x8000C0C0; // cyan semi-transparent
            int transrvb = 0x00000000;
            int ix1, ix2;
            double ang, angmin;
            for (int iy=0; iy<himg/2; iy++) {
                if (iy < himg/2 - hmin/2 || iy > himg/2 + hmin/2)
                    ix1 = 0;
                else {
                    angmin = Math.acos(((double)(iy - himg/2))/(hmin/2));
                    ix1 = (int)Math.round( Math.sin(angmin) * (wmin/2) );
                }
                ang = Math.acos(((double)(iy - himg/2))/(h/2));
                ix2 = (int)Math.round( Math.sin(ang) * (w/2) );
                if (ix2 > wimg/2)
                    ix2 = wimg/2;
                for (int ix=0; ix<wimg/2-ix2; ix++)
                    pix[index++] = transrvb;
                if (ix2 > ix1)
                    for (int ix=wimg/2-ix2; ix<wimg/2-ix1; ix++)
                        pix[index++] = rvb;
                for (int ix=wimg/2-ix1; ix<wimg/2; ix++)
                    pix[index++] = transrvb;
                for (int ix=wimg/2; ix<wimg/2+ix1; ix++)
                    pix[index++] = transrvb;
                if (ix2 > ix1)
                    for (int ix=wimg/2+ix1; ix<wimg/2+ix2; ix++)
                        pix[index++] = rvb;
                for (int ix=wimg/2+ix2; ix<wimg; ix++)
                    pix[index++] = transrvb;
            }
            imgzonehab1 = createImage(new MemoryImageSource(wimg, himg/2, pix, 0, wimg));
            pix = new int[wimg * himg / 2];
            index = 0;
            for (int iy=himg/2; iy<himg; iy++) {
                if (iy < himg/2 - hmin/2 || iy > himg/2 + hmin/2)
                    ix1 = 0;
                else {
                    angmin = Math.acos(((double)(iy - himg/2))/(hmin/2));
                    ix1 = (int)Math.round( Math.sin(angmin) * (wmin/2) );
                }
                ang = Math.acos(((double)(iy - himg/2))/(h/2));
                ix2 = (int)Math.round( Math.sin(ang) * (w/2) );
                if (ix2 > wimg/2)
                    ix2 = wimg/2;
                for (int ix=0; ix<wimg/2-ix2; ix++)
                    pix[index++] = transrvb;
                if (ix2 > ix1)
                    for (int ix=wimg/2-ix2; ix<wimg/2-ix1; ix++)
                        pix[index++] = rvb;
                for (int ix=wimg/2-ix1; ix<wimg/2; ix++)
                    pix[index++] = transrvb;
                for (int ix=wimg/2; ix<wimg/2+ix1; ix++)
                    pix[index++] = transrvb;
                if (ix2 > ix1)
                    for (int ix=wimg/2+ix1; ix<wimg/2+ix2; ix++)
                        pix[index++] = rvb;
                for (int ix=wimg/2+ix2; ix<wimg; ix++)
                    pix[index++] = transrvb;
            }
            imgzonehab2 = createImage(new MemoryImageSource(wimg, himg/2, pix, 0, wimg));
            wminimgzonehab = wmin;
        }
        if (devant)
            g.drawImage(imgzonehab2, imgdimx/2 - wimg/2, imgdimy/2, null);
        else
            g.drawImage(imgzonehab1, imgdimx/2 - wimg/2, imgdimy/2 - himg/2, null);
    }
    
}
