/*
Copyright (C) 2005 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package orbites;

import java.awt.*;
import java.awt.image.MemoryImageSource;
import java.util.Random;

/**
 * Reprsente une plante tournant autour d'une toile.
 */
public class Planete {
    
    private static final double mSoleil = 1.9887973E30; // masse du Soleil (en kg)
    private static final double mJupiter = 1.900E27; // masse de Jupiter (en kg)
    private static final double mTerre = 5.9736E24; // masse de la Terre (en kg)
    private static final double Rj = 7.1492E7; // rayon quatorial de Jupiter (en m)
    private static final double Rt = 6.37814E6; // rayon quatorial de la Terre (en m)
    private static final double Mt = 5.9733E24; // masse de la Terre (en kg)
    private static final double UA = 1.49597870691E11; // une unit astronomique (en m)
    private Orbites orbites;
    private int imgdimx;
    private int imgdimy;
    private String nom;
    private double dgap, periode, eccentricite, inc, msini, omega;
    private double inc0; // sauvegarde valeur inclinaison (utilise avec l'option inclinaisonNulle)
    private boolean incConnue, eccConnue, perConnue;
    private double rpl; // rayon (en m)
    private double tpstheta;
    private Image imgPlanete;
    private String typePlanete;
    private double rorb;
    private double r2; // rayon ajust si tailleReelle = false
    private long seed;
    int posx, posy; // coordonnes en pixels
    
    public Planete(Orbites orbites) {
        this.orbites = orbites;
    }
    
    public double getDgap() {
        return(dgap);
    }
    
    public double getPeriode() {
        return(periode);
    }
    
    public double getEccentricite() {
        return(eccentricite);
    }
    
    public double getInc() {
        return(inc);
    }
    
    public double getMasse() {
        return(msini);
    }
    
    public boolean getIncConnue() {
        return(incConnue);
    }
    
    public double getOmega() {
        return(omega);
    }
    
    public double getRpl() {
        return(rpl);
    }
    
    public double getRorb() {
        return(rorb);
    }
    
    public void setRorb(double rorb) {
        this.rorb = rorb;
    }
    
    public double getTpstheta() {
        return(tpstheta);
    }
    
    public Image getImage() {
        return(imgPlanete);
    }
    
    /** temprature de la plante, en supposant qu'elle tourne autour de l'toile donne en paramtre */
    public double getTemperature(Etoile etoile) {
        // Tpl = Tet*(Ret/2d)^1/2 * (1-A)^1/4 * G
        // on ignore A et G ici
        // variation avec la masse ? (pifomtrique)
        // il faudrait aussi tenir compte de l'ge de la plante pour bien faire
        double mplanete;
        if (Math.abs(inc-Math.PI/2) > 0.01)
            mplanete = msini / Math.sin(Math.PI/2 - inc);
        else
            mplanete = msini;
        return(etoile.getTemperature() * Math.sqrt(etoile.getRayon() / (2 * dgap)) + 20*(mplanete/mJupiter-1));
    }
    
    public void init(Etoile etoile, boolean activerImages, double dhabmin, double dhabmax, int imgdimx, int imgdimy) {
        this.imgdimx = imgdimx;
        this.imgdimy = imgdimy;
        
        if (!perConnue)
            periode = Math.sqrt(Math.pow(dgap / UA, 3)/(etoile.getMasse()/mSoleil)) * 365.25;
        
        seed = System.currentTimeMillis();
        
        String suffixe;
        if ("1.2".compareTo(System.getProperty("java.version")) < 0)
            suffixe = ".png";
        else
            suffixe = ".gif";
        
        double mplanete;
        if (Math.abs(inc-Math.PI/2) > 0.0001)
            mplanete = msini / Math.sin(Math.PI/2 - inc);
        else
            mplanete = msini;
        
        if (mplanete > mTerre*12.5) {  // limite prcise ?
            double temp = getTemperature(etoile);
            String snum;
            if (temp > 50 && temp < 150 || temp > 1500) {
                Random rnd = new Random(seed);
                if (rnd.nextDouble() < 0.5)
                    snum = "1";
                else
                    snum = "1b";
            } else if (temp > 50 && temp < 350)
                snum = "2";
            else if (temp < 900)
                snum = "3";
            else
                snum = "4";
            typePlanete = "gazeuse" + snum;
        } else if (dhabmin < dgap && dgap < dhabmax && eccentricite < 0.3)
            typePlanete = "habitable1";
        else
            typePlanete = "tellurique1";
        if (rpl == 0) { // si le rayon n'est pas donn, on en fait une estimation
            if (typePlanete.indexOf("gazeuse") == 0)
                rpl = Rj;
            else
                rpl = Rt*Math.pow(mplanete / Mt, 0.2);
        }
        if (activerImages && imgPlanete == null)
            imgPlanete = orbites.getImage(orbites.getDocumentBase(), "images/" + typePlanete + suffixe);
        
        tpstheta = 0;
        rorb = dgap;
    }
    
    /**
     * Lecture des paramtres pour la plante avec le numro donn
     */
    protected void lectureParametres(int ipl) throws NumberFormatException {
        nom = orbites.getParameter("nom"+ipl);
        String sa = orbites.getParameter("a"+ipl);
        if (sa != null && !"".equals(sa) && !"0".equals(sa))
            dgap = Double.valueOf(sa).doubleValue() * UA;
        else
            throw new NumberFormatException(Messages.get("demi-grand_axe_inconnu"));
        String speriode = orbites.getParameter("T"+ipl);
        if (speriode != null && !"".equals(speriode)) {
            perConnue = true;
            periode = Double.valueOf(speriode).doubleValue();
        } else
            perConnue = false;
        String se = orbites.getParameter("e"+ipl);
        if (se != null && !"".equals(se)) {
            eccConnue = true;
            eccentricite = Double.valueOf(se).doubleValue();
        } else {
            eccConnue = false;
            eccentricite = 0;
        }
        String somega = orbites.getParameter("omega"+ipl);
        if (somega != null && !"".equals(somega))
            omega = Double.valueOf(somega).doubleValue() * Math.PI/180;
        else
            omega = 0;
        String si = orbites.getParameter("i"+ipl);
        if (si != null && !"".equals(si)) {
            incConnue = true;
            inc = Math.abs(90-Double.valueOf(si).doubleValue()) * Math.PI/180;
        } else {
            incConnue = false;
            inc = (90 - 0) * Math.PI/180; // on prend une inclinaison de 0
            // (pour bien voir l'orbite, malgr que les mthodes de dtections favorisent des inclinaisons plus grandes)
        }
        inc0 = inc;
        String smsini = orbites.getParameter("msini"+ipl);
        if (smsini != null && !"".equals(smsini))
            msini = Double.valueOf(smsini).doubleValue() * mJupiter;
        else
            throw new NumberFormatException(Messages.get("masse_planete_inconnue"));
        String srpl = orbites.getParameter("rayon"+ipl);
        if (srpl != null && !"".equals(srpl) && !"0".equals(srpl))
            rpl = Double.valueOf(srpl).doubleValue() * Rt;
        else
            rpl = 0;
        String simage = orbites.getParameter("image"+ipl);
        if (simage != null && !simage.equals("")) {
            String suffixe;
            if ("1.2".compareTo(System.getProperty("java.version")) < 0)
                suffixe = ".png";
            else
                suffixe = ".gif";
            imgPlanete = orbites.getImage(orbites.getDocumentBase(), simage + suffixe);
        } else
            imgPlanete = null;
    }
    
    public void iteration(double interv) {
        int nprecision = 10;
        
        double p2 = rorb * (1 - Math.pow(eccentricite, 2));
        double rr2 = p2 / (1 + eccentricite * Math.cos(tpstheta));
        double reccentricite = Math.sqrt(1 - Math.pow(eccentricite, 2));
        double rorb22 = Math.pow(rorb, 2);
        for (int i=0; i<nprecision; i++) {
            tpstheta += interv * rorb22 * reccentricite / Math.pow(rr2, 2);
            rr2 = p2 / (1 + eccentricite * Math.cos(tpstheta));
        }
        while (tpstheta > 2*Math.PI)
            tpstheta -= 2*Math.PI;
    }
    
    public double[] calculPos() {
        double p2 = rorb * (1 - Math.pow(eccentricite, 2));
        double rr2 = p2 / (1 + eccentricite * Math.cos(tpstheta));
        double x2 = rr2 * Math.cos(tpstheta + omega);
        double y2 = rr2 * Math.sin(tpstheta + omega) * Math.sin(inc);
        double[] pos = new double[2];
        pos[0] = x2;
        pos[1] = y2;
        return(pos);
    }
    
    public void setInclinaisonNulle(boolean inclinaisonNulle) {
        if (inclinaisonNulle)
            inc = Math.PI/2;
        else
            inc = inc0;
    }
    
    public void rotation(double rotvert) {
        inc += rotvert;
    }
    
    public void resetRotation() {
        inc = inc0;
    }
    
    public double distsel(int x, int y) {
        return(Math.sqrt(Math.pow(x-posx,2) + Math.pow(y-posy,2)));
    }
    
    protected void majCroquis(Graphics g, double rapport, double maxrorb, double r1,
            boolean tailleReelle, boolean activerImages) {
        
        if (tailleReelle)
            r2 = rpl;
        else
            r2 = r1/3;
        
        // (rr2, tpstheta + omega) : coordonnes polaires de la plante
        // (x2, y2) : coordonnes cartsiennes de la plante
        double p2 = rorb * (1 - Math.pow(eccentricite, 2));
        double rr2 = p2 / (1 + eccentricite * Math.cos(tpstheta));
        double x2 = rr2 * Math.cos(tpstheta + omega);
        double y2 = rr2 * Math.sin(tpstheta + omega) * Math.sin(inc);
        
        double alpha = 0;
        if (tpstheta + omega < Math.PI/2)
            alpha = tpstheta + omega + Math.PI/2;
        else if (tpstheta + omega >= Math.PI/2 && tpstheta + omega < Math.PI)
            alpha = 3*Math.PI/2 - (tpstheta + omega);
        else if (tpstheta + omega >= Math.PI && tpstheta + omega < 3*Math.PI/2)
            alpha = 3*Math.PI/2 - (tpstheta + omega);
        else if (tpstheta + omega >= 3*Math.PI/2)
            alpha = tpstheta + omega - 3*Math.PI/2;
        // on se plaCAIT  une distance du centre de masse de 2*orb2
        // maintenant (avec plusieurs orbites): on utilise 2*maxrorb  la place
        // en supposant l'orbite sphrique, le cosinus de l'angle entre la plante et l'observateur
        // par rapport au centre de masse vaut cos(alpha)*cos(90-i)
        // dobs^2 = rr2^2 + (2*rorb2)^2 - 2*rr2*(2*rorb2)*cos(alpha)*cos(inc)
        double dobs;
        // dobs faux si inc != 0 et ecc != 0
        dobs = Math.sqrt(rr2*rr2 + Math.pow(2*maxrorb, 2) - 2*rr2*(2*maxrorb)*Math.cos(alpha)*Math.cos(inc));
        
        int ix2, iy2, id2;
        //int ir2 = (int)Math.round(r2*rapport);
        int ir2 = (int)Math.round(r2*rapport/(dobs/(2*maxrorb))+0.5);
        //int id2 = (int)Math.round(2*r2*rapport);
        id2 = (int)Math.round(2*r2*rapport/(dobs/(2*maxrorb))+1);
        posx = (int)Math.round(imgdimx/2 + x2*rapport);
        ix2 = (int)Math.round(imgdimx/2 + x2*rapport) - ir2;
        posy = (int)Math.round(imgdimy/2 - y2*rapport);
        iy2 = (int)Math.round(imgdimy/2 - y2*rapport) - ir2;
        
        if (activerImages)
            imagePlanete(g, ix2, iy2, id2, id2);
        else
            dessinPlanete(g, ix2, iy2, id2, id2);
    }
    
    protected void afficherInfos(Graphics g, int post) {
        g.setColor(Color.white);
        if (nom == null || "".equals(nom))
            g.drawString(Messages.get("planete_selectionnee") + " " + Messages.get("?"), 5, post+=12);
        else
            g.drawString(Messages.get("planete_selectionnee") + " " + nom, 5, post+=12);
        if (!incConnue)
            g.drawString(Messages.get("inclinaison_inconnue") + " " +
                joliFormattage(90-inc*180/Math.PI) + "", 5 , post+=12);
        if (!eccConnue)
            g.drawString(Messages.get("excentricite_inconnue"), 5 , post+=12);
        if (!perConnue)
            g.drawString(Messages.get("periode_inconnue"), 5 , post+=12);
        //g.drawString("a = " + joliFormattage(dgap/UA) + " UA", 5, post+=12);
        /*
        if (!incConnue)
            g.drawString("m > " + joliFormattage(msini/mJupiter) + " Mjup", 5, post+=12);
        else
            g.drawString("m = " + joliFormattage(msini/(Math.sin(Math.PI/2-inc)*mJupiter)) +
                " Mjup", 5, post+=12);
        */
        g.drawString(Messages.get("periode") + " " + joliFormattage(periode) + " " + Messages.get("jours"), 5, post+=12);
        //g.drawString("fps : " + joliFormattage(1000/temporisation) + " images/s", 5, post+=12);
    }
    
    protected void afficherParametres(Graphics g, double x2, double y2, double rapport,
        boolean affdga, boolean affecc, boolean affinc, boolean affmasses, boolean affomega) {
        
        g.setColor(Color.green);
        
        if (affdga || affecc) {
            int lx1 = (int)Math.round(imgdimx/2 - rorb*rapport - rorb*eccentricite*rapport);
            int ly1 = (int)Math.round(imgdimy/2);
            int lx2 = (int)Math.round(imgdimx/2 - rorb*eccentricite*rapport);
            int ly2 = ly1;
            
            if (omega != 0) {
                double r = rorb + rorb*eccentricite;
                double dlx1 = r * Math.cos(Math.PI + omega);
                double dly1 = r * Math.sin(Math.PI + omega) * Math.sin(inc);
                lx1 = (int)Math.round(imgdimx/2 + dlx1*rapport);
                ly1 = (int)Math.round(imgdimy/2 - dly1*rapport);
                
                r = -rorb*eccentricite;
                double dlx2 = r * Math.cos(omega);
                double dly2 = r * Math.sin(omega) * Math.sin(inc);
                lx2 = (int)Math.round(imgdimx/2 + dlx2*rapport);
                ly2 = (int)Math.round(imgdimy/2 - dly2*rapport);
            }
            
            dessinerFleche(g, lx1, ly1, lx2, ly2);
            if (affdga) {
                g.setColor(Color.black);
                String s = "a = " + joliFormattage(dgap/UA) + " " + Messages.get("UA");
                int sw = g.getFontMetrics().stringWidth(s);
                int sx = (lx1+lx2)/2 - sw/2;
                g.fillRect(sx, (ly1+ly2)/2 + 2, sw, 12);
                g.setColor(Color.green);
                g.drawString(s, sx, (ly1+ly2)/2 + 12);
            } else
                g.drawString("a", (lx1+lx2)/2, (ly1+ly2)/2+12);
            
            if (affecc) {
                int lx3 = (int)Math.round(imgdimx/2);
                int ly3 = (int)Math.round(imgdimy/2);
                g.setColor(new Color(50, 100, 255));
                if (eccentricite != 0)
                    dessinerFleche(g, lx2, ly2, lx3, ly3);
                g.drawString("c", (lx3+lx2)/2, (ly2+ly3)/2+12);
                g.setColor(new Color(255, 200, 0));
                g.drawString("e = c/a = " + joliFormattage(eccentricite),
                    (lx1+lx3)/2-20, (ly3+ly1)/2+40);
                g.setColor(Color.green);
            }
        }
        
        if (affinc) {
            int cx1 = (int)Math.round(imgdimx/2);
            int cy1 = (int)Math.round(imgdimy/2);
            g.drawOval(cx1-(int)Math.round(rorb*rapport), cy1-(int)Math.round(rorb*rapport),
                (int)Math.round(rorb*rapport*2), (int)Math.round(rorb*rapport*2));
            g.drawOval(cx1-(int)Math.round(rorb*rapport),
                cy1-(int)Math.round(rorb*rapport*Math.sin(inc)),
                (int)Math.round(rorb*rapport*2), (int)Math.round(rorb*rapport*Math.sin(inc)*2));
            
            double rc = 50;
            double d = rorb*rapport;
            double axa = (rc*rc - 2*d*d)/(2*d) + d;
            //System.out.println("axa ="+ axa);
            double re = rorb*rapport*Math.sin(inc);
            double a = 1 - re*re/(d*d);
            double b = 2*d;
            double c = d*d + re*re - rc*rc;
            double rdelta = Math.sqrt(b*b - 4*a*c);
            //System.out.println("rdelta ="+ rdelta);
            //double axb1 = (-b - rdelta)/(2*a) + d;
            double axb2 = (-b + rdelta)/(2*a) + d;
            //System.out.println(axb1 + " " + axb2);
            double angle1 = Math.acos(axa/rc);
            int iangle1 = (int)Math.round(angle1*180/Math.PI);
            double angle2 = Math.acos(axb2/rc);
            int iangle2 = (int)Math.round(angle2*180/Math.PI);
            int iax1 = cx1 - (int)Math.round(d) - (int)rc;
            int iay1 = cy1 - (int)rc;
            int iax2 = cx1 - (int)Math.round(d) + (int)rc;
            int iay2 = cy1 + (int)rc;
            
            g.drawArc(iax1, iay1, iax2 - iax1, iay2 - iay1, iangle2, iangle1-iangle2);
            int iaxa = cx1 - (int)Math.round(d) + (int)Math.round(axa) + 10;
            double aya = Math.sqrt(d*d - (axa-d)*(axa-d));
            int iaya = cy1 - (int)Math.round(aya);
            g.drawString("inc = " + joliFormattage(90 - inc0*180/Math.PI)+"", iaxa, iaya);
        }
        
        if (affmasses) {
            int xl = (int)Math.round(imgdimx/2 + x2*rapport) + 10;
            int yl = (int)Math.round(imgdimy/2 - y2*rapport);
            if (!incConnue) {
                if (msini > 12.5*mTerre)
                    g.drawString("m > " + joliFormattage(msini/mJupiter) + " " + Messages.get("Mjup"), xl, yl);
                else
                    g.drawString("m > " + joliFormattage(msini/mTerre) + " " + Messages.get("Mterre"), xl, yl);
            } else {
                double m2;
                if (Math.abs(Math.PI/2-inc0) < 1E-8)
                    m2 = msini;
                else
                    m2 = msini/Math.sin(Math.PI/2-inc0);
                if (m2 > 12.5*mTerre)
                    g.drawString("m = " + joliFormattage(m2/mJupiter) + " " + Messages.get("Mjup"), xl, yl);
                else
                    g.drawString("m = " + joliFormattage(m2/mTerre) + " " + Messages.get("Mterre"), xl, yl);
            }
        }
        
        if (affomega) {
            // centre de masse
            int cx = (int)Math.round(imgdimx/2);
            int cy = (int)Math.round(imgdimy/2);
            // ligne des noeuds
            g.drawLine(0, cy, imgdimx, cy);
            String s = Messages.get("ligne_des_noeuds");
            g.drawString(s, imgdimx - g.getFontMetrics().stringWidth(s) - 10, cy - 5);
            // priastre
            double p2 = rorb * (1 - Math.pow(eccentricite, 2));
            double rri = p2 / (1 + eccentricite);
            double xi = rri * Math.cos(omega);
            double yi = rri * Math.sin(omega) * Math.sin(inc);
            int irri = (int)Math.round(rri*rapport);
            int ixi = (int)Math.round(cx + xi*rapport);
            int iyi = (int)Math.round(cy - yi*rapport);
            g.drawLine(cx, cy, ixi, iyi);
            g.drawString(Messages.get("periastre"), ixi + 2, iyi + 5);
            // omega
            int iomega = (int)Math.round(omega*180/Math.PI);
            int idy = (int)Math.round((irri/2.) * Math.sin(inc));
            g.drawArc(cx-irri/4, cy-idy/2, irri/2, idy, 0, iomega);
            int ixtxt = (int)Math.round(cx + (rri/4) * Math.cos(omega/2) * rapport);
            int iytxt = (int)Math.round(cy - (rri/4) * Math.sin(omega/2) * Math.sin(inc) * rapport);
            if (iytxt > cy && iytxt < cy + 5)
                iytxt = cy + 5;
            else if (iytxt < cy && iytxt > cy - 7)
                iytxt = cy - 7;
            g.drawString("omega = " + joliFormattage(omega*180/Math.PI) + "", ixtxt + 3, iytxt + 5);
        }
    }
    
    /**
     * dessin d'une flche double
     */
    protected void dessinerFleche(Graphics g, int x1, int y1, int x2, int y2) {
        g.drawLine(x1, y1, x2, y2);
        if (x2 == x1)
            return;
        double angle = Math.atan(Math.abs(((double)(y2-y1))/(x2-x1)));
        if (x2 < x1)
            angle = Math.PI - angle;
        if (y2 < y1)
            angle = -angle;
        double angle1 = angle - Math.PI/12;
        double angle2 = angle + Math.PI/12;
        double lg = Math.sqrt(Math.pow(x2-x1,2) + Math.pow(y2-y1,2)) / 15;
        if (lg < 4)
            lg = 4;
        int fx1, fy1, fx2, fy2;
        fx1 = x2;
        fy1 = y2;
        fx2 = (int)Math.round(fx1 - lg*Math.cos(angle1));
        fy2 = (int)Math.round(fy1 - lg*Math.sin(angle1));
        g.drawLine(fx1, fy1, fx2, fy2);
        fx2 = (int)Math.round(fx1 - lg*Math.cos(angle2));
        fy2 = (int)Math.round(fy1 - lg*Math.sin(angle2));
        g.drawLine(fx1, fy1, fx2, fy2);
        fx1 = x1;
        fy1 = y1;
        fx2 = (int)Math.round(fx1 + lg*Math.cos(angle1));
        fy2 = (int)Math.round(fy1 + lg*Math.sin(angle1));
        g.drawLine(fx1, fy1, fx2, fy2);
        fx2 = (int)Math.round(fx1 + lg*Math.cos(angle2));
        fy2 = (int)Math.round(fy1 + lg*Math.sin(angle2));
        g.drawLine(fx1, fy1, fx2, fy2);
    }
    
    protected void dessinOrbite(Graphics g, double rapport, boolean devant) {
        double tpsdo;
        g.setColor(Color.red);
        int ndo = 100;
        int nprecision = 10;
        double tpsdotheta = 0;
        double p2 = rorb * (1 - Math.pow(eccentricite, 2));
        double reccentricite = Math.sqrt(1 - Math.pow(eccentricite, 2));
        double interv = 2*Math.PI / (ndo * nprecision);
        double rorb22 = Math.pow(rorb, 2);
        for (int itpsdo=0; itpsdo<ndo; itpsdo++) {
            double rr2 = p2 / (1 + eccentricite * Math.cos(tpsdotheta));
            double x2 = rr2 * Math.cos(tpsdotheta+omega);
            double y2 = rr2 * Math.sin(tpsdotheta+omega) * Math.sin(inc);
            // intgration de l'angle theta: plusieurs itrations pour plus de prcision
            for (int i=0; i<nprecision; i++) {
                tpsdotheta += interv * rorb22 * reccentricite / Math.pow(rr2, 2);
                rr2 = p2 / (1 + eccentricite * Math.cos(tpsdotheta));
            }
            boolean testinc = (Math.sin(2*inc) > 0);
            boolean retourne = (y2 < 0 && testinc || y2 >= 0 && !testinc);
            if ((devant && retourne) || (!devant && !retourne)) {
                int ix2 = (int)Math.round(imgdimx/2 + x2*rapport);
                int iy2 = (int)Math.round(imgdimy/2 - y2*rapport);
                g.drawRect(ix2, iy2, 1, 1);
            }
        }
    }
    
    protected void dessinPlanete(Graphics g, int x, int y, int longueur, int hauteur) {
        Color couleurPlanete = new Color(230, 150, 50); // pas plus de 240 par couleur !
        g.setColor(couleurPlanete);
        g.fillOval(x, y, longueur, hauteur);
        /*
        // pastille
        g.setColor(Color.red);
        int longueur2 = (int)Math.round(longueur * Math.abs(Math.sin(tps*10)));
        g.fillOval(x + (longueur - longueur2)/2 , y, longueur2, hauteur);
        g.setColor(new Color(200, 0, 0));
        g.fillOval(x + (longueur - longueur2)/2 + longueur2/4, y + hauteur/4, longueur2/2, hauteur/2);
        g.drawOval(x + (longueur - longueur2)/2, y, longueur2, hauteur);
        */
        int rouge = couleurPlanete.getRed();
        int vert = couleurPlanete.getGreen();
        int bleu = couleurPlanete.getBlue();
        Color couleur;
        Random rnd = new Random(seed);
        for (int yp=hauteur-1; yp>0; yp--) {
            double dy = Math.abs(hauteur/2.0 - yp);
            double dw = Math.sqrt(Math.pow(hauteur/2.0, 2) - dy*dy) - 0.5;
            int xp1 = (int)Math.round(x + longueur/2.0 - dw);
            int xp2 = (int)Math.round(x + longueur/2.0 + dw);
            couleur = new Color(rouge + (int)(rnd.nextDouble()*20) - 10,
                vert + (int)(rnd.nextDouble()*20) - 10,
                bleu + (int)(rnd.nextDouble()*20) - 10);
            g.setColor(couleur);
            if (dw > 0)
                g.fillOval(xp1, y + yp - (int)(dw/8), xp2 - xp1, (int)(dw/4));
        }
    }
    
    protected void imagePlanete(Graphics g, int x, int y, int longueur, int hauteur) {
        if (imgPlanete == null) {
            dessinPlanete(g, x, y, longueur, hauteur);
            ombrePlanete(g, x, y, longueur, hauteur);
            return;
        }
        double wpl, hpl;
        if (typePlanete.indexOf("gazeuse") == 0) {
            wpl = imgPlanete.getWidth(orbites) / 98.; // la plante doit tenir en 100x100 sur l'image
            hpl = imgPlanete.getHeight(orbites) / 98.;
        } else {
            wpl = imgPlanete.getWidth(orbites) / 100.;
            hpl = imgPlanete.getHeight(orbites) / 100.;
        }
        int w2 = (int)Math.round(longueur * wpl)+1; // 4x2 sans le +1 -> pas affich
        int h2 = (int)Math.round(hauteur * hpl)+1;
        int x2 = x + longueur/2 - w2/2;
        int y2 = y + hauteur/2 - h2/2;
        g.drawImage(imgPlanete, x2, y2, w2, h2, orbites);
        if (imgPlanete.getWidth(orbites) == 100)
            ombrePlanete(g, x2-1, y2-1, w2+2, h2+2);
        else { // cas de plante avec anneaux: le disque est plus petit que l'image
            wpl = 100. / 98; // le disque doit tre en 100x100
            hpl = 100. / 98;
            w2 = (int)Math.round(longueur * wpl)+1;
            h2 = (int)Math.round(hauteur * hpl)+1;
            x2 = x + longueur/2 - w2/2;
            y2 = y + hauteur/2 - h2/2;
            ombrePlanete(g, x2-1, y2-1, w2+2, h2+2);
        }
    }
    
    protected void ombrePlanete(Graphics g, int x, int y, int longueur, int hauteur) {
        double angle = 3 * Math.PI - (tpstheta + omega);
        if (angle < 0)
            angle += 2*Math.PI;
        if (angle > 2*Math.PI)
            angle -= 2*Math.PI;
        if (angle > 2*Math.PI)
            angle -= 2*Math.PI;
        boolean inv = (angle > Math.PI && inc > 0 || angle < Math.PI && inc < 0);
        if (inv) {
            angle = angle + Math.PI;
            if (angle > 2*Math.PI)
                angle -= 2*Math.PI;
        }
        int rayon = longueur/2;
        angle = Math.atan(Math.tan(angle)*Math.sin(inc));
        if (angle < 0)
            angle += 2*Math.PI;
        double angle1 = angle + Math.PI / 2; // angle de position de l'toile par rapport  la plante
        if (angle1 > 2*Math.PI)
            angle1 -= 2*Math.PI;
        double a = Math.tan(angle1);
        double b = rayon * (1 - a);
        double den = Math.sqrt(a*a + 1);
        double a2 = Math.tan(angle1-Math.PI/2);
        double b2 = rayon * (1 - a2);
        double den2 = Math.sqrt(a2*a2 + 1);
        double d1, d2, d3;
        double yt;
        double encoreuntruc = Math.abs(Math.sin(tpstheta + omega)) * Math.cos(inc); // quantit d'ombre de 0  1
        if (inc < 0)
            encoreuntruc = -encoreuntruc;
        long deltx, delty;
        double dist;
        long distc;
        long maxdistc = rayon * rayon + 4;
        int alpha;
        int[] pix = new int[longueur * hauteur];
        int index = 0;
        for (int iy=0; iy<hauteur; iy++)
            for (int ix=0; ix<longueur; ix++) {
                deltx = ix - rayon;
                delty = iy - rayon;
                distc = deltx * deltx + delty * delty;
                if (distc < maxdistc) {
                    yt = a * ix + b;
                    d1 = Math.abs((yt - iy) / den);
                    d2 = Math.abs((a2*ix - iy + b2) / den2);
                    d3 = Math.sqrt(rayon*rayon - d2*d2);
                    if (d3 == 0)
                        dist = 0;
                    else
                        dist = d1 / d3;
                    if (yt > iy)
                        dist = -dist;
                    dist = dist + encoreuntruc;
                    alpha = (int)(128 + 128 * dist);
                    if (alpha < 0)
                        alpha = 0;
                    if (alpha > 255)
                        alpha = 255;
                    if (!inv)
                        alpha = 255 - alpha;
                    pix[index++] = alpha << 24;
                } else
                    pix[index++] = 0;
            }
        Image imgombre = orbites.createImage(new MemoryImageSource(longueur, hauteur, pix, 0, longueur));
        g.drawImage(imgombre, x, y, longueur, hauteur, orbites);
    }
    
	// mthode d'affichage de double issue du Calcotron
    protected static String joliFormattage(double v) {
        int precision = 3;
        int limitesci = 3; // dcimales avant de passer en mode sci
        
        if (v == 0)
            return("0");
        String sj = ""+v;
        if ("Infinity".equals(sj) || "-Infinity".equals(sj) || "NaN".equals(sj))
            return(""+v);
        String signe;
        if (v >= 0)
            signe = "";
        else {
            v = -v;
            signe = "-";
        }
        int exposant = (int)Math.floor(Math.log(v)/Math.log(10));
        double exparr = Math.pow(10, precision);
        double mantisse = v / Math.pow(10, exposant);
        mantisse = Math.rint(mantisse*exparr)/exparr;
        if (mantisse >= 10) {
            exposant += 1;
            mantisse = v / Math.pow(10, exposant);
            mantisse = Math.rint(mantisse*exparr)/exparr;
        }
        String sv = "";
        int chiffre = (int)Math.floor(mantisse);
        sv += chiffre + ".";
        for (int i=0; i<precision-1; i++) {
            mantisse = (mantisse - chiffre)*10;
            double exparr2 = Math.pow(10, precision-i);
            mantisse = Math.rint(mantisse*exparr)/exparr;
            chiffre = (int)Math.floor(mantisse);
            sv += chiffre;
        }
        mantisse = (mantisse - chiffre)*10;
        chiffre = (int)Math.round(mantisse);
        sv += chiffre;
        char dernier = sv.charAt(sv.length()-1);
        while (dernier == '0') {
            sv = sv.substring(0, sv.length()-1);
            dernier = sv.charAt(sv.length()-1);
        }
        if (exposant != 0) {
            if (exposant > 0 && exposant < limitesci) {
                String sd = sv.substring(0,1);
                if (sv.length()-2 < exposant) {
                    // transformation de 3.2E2 en 320
                    if (sv.length() > 2)
                        sd += sv.substring(2,sv.length());
                    for (int i=0; i<exposant-(sv.length()-2); i++)
                        sd += '0';
                } else {
                    // transformation de 3.245E2 en 324.5
                    if (sv.length() < exposant+2)
                        sd += sv.substring(2);
                    else
                        sd += sv.substring(2,exposant+2) + "." + sv.substring(exposant+2);
                }
                sv = sd;
            } else if (exposant < 0 && exposant > -limitesci) {
                // transformation de 3.2E-2 en 0.032
                String sd = "0.";
                for (int i=0; i<-exposant-1; i++)
                    sd += '0';
                sv = sd + sv.charAt(0) + sv.substring(2);
            } else {
                if (sv.charAt(sv.length()-1) == '.')
                    sv = sv.substring(0, sv.length()-1);
                sv += ".10^" + exposant;
            }
        }
        if (sv.charAt(sv.length()-1) == '.')
            sv = sv.substring(0, sv.length()-1);
        
        sv = signe + sv;
        
        return(sv);
    }
    
}
